
# Script for Imaging Arp220 ALMA B5 Science Verification Data
# Run with casapy_4.7.0-1
#
# List of steps in this Imaging script
#

DoSelfCal=T              # This will self calibrate Arp220.
DoContSubtract=T	 # This will subtract the continuum from the visibility data
DoCleanAllspw=T          # This will clean all the spectral windows, in channel mode.
DoCleanLine=T            # This will clean identified lines in velocity mode.
DoCleanCont=T            # This will clean the continuum of Arp220.
DoCleanCheckSou=F        # This will image de check source J1532+2344, per spw.




VISINAME='uid___A002_Xb56f17_X164.ms.split.cal'
SOUNAME='Arp220'
TUNING='B5'

phasecenterArp220='J2000 15h34m57.22 +23d30m11.6'
maskArp220=['circle[ [15h34m57.223,+23d30m11.53], 2.0arcsec]','circle[ [15h34m57.293,+23d30m11.38], 1.5arcsec]']
imsize=[600,600]
cell='0.1arcsec'




spw4Cont='2:20~400;1500~1900,3:20~700;1300~1900'  
# This is list of line free channels for self-cal and continuum subtraction:
# Difficult to find any obvious line-free channels in spw 0,1. The continuum is thus extrapolated from spw 2,3.
# Note that there seems to be some weak lines  within this channel selection. This affects the continuum subtraction for the bright western core (the continuum is overestimated). The spectral baseline of the eastern core looks ok. This is not a problem for the self-calibration procedure.




if DoSelfCal: # SELF-CAL

	
	os.system('rm -rf '+VISINAME+'.flagversions')
	clearcal(vis=VISINAME,field='Arp_220')

    	print "Clean CONT 4 selfcal"
    	os.system('rm -rf '+SOUNAME+'-cont_preSelfCal*')
    	tclean(vis=VISINAME,
    	      imagename=SOUNAME+'-cont_preSelfCal',
	      datacolumn='data',
    	      field='Arp_220',
    	      spw=spw4Cont,
    	      specmode='mfs',
    	      niter=100,
    	      threshold='10mJy',
    	      interactive=False,
    	      mask=maskArp220,
    	      imsize=imsize,
    	      cell=cell,
    	      weighting='briggs',
    	      robust=0.5,
    	      phasecenter=phasecenterArp220,
	      savemodel='modelcolumn') # Do not forget that to fill in the model in the model column !

    	print "SELFCAL Phase"
    	os.system('rm -rf table-selfcal.scp1')
    	gaincal(vis = VISINAME,
    	        field='Arp_220',
    	        caltable='table-selfcal.scp1',
    	        solint='int', # time span for self-cal phase gain solutions (here set per integration) 
    	        refant='DA61',
    	        spw=spw4Cont,
		combine='spw',	# will combine all spw in spw4Cont selection to determine the solutions
    	        calmode='p',  # will determine gain phase solutions only
    	        gaintype='T')

    	plotcal(caltable='table-selfcal.scp1', xaxis='time', yaxis='phase', plotrange=[0,0,-180,180], iteration='spw', subplot = 221) # will plot selfcal gain solutions


    	print "Applying SELFCAL solutions"
    	applycal(vis=VISINAME,
    	         field='Arp_220',
    	         calwt=False,
		 gaintable=['table-selfcal.scp1'],
		 spwmap=[2,2,2,2],  # this will apply selfcal gain phase solutions derived above (combining all swp in spw4Cont). Since we combined solutions over spectral window (combine='spw'), there is only one gain solution column stored in spw=2 This is applied to all of them individually.
		 applymode='calonly'
		)    	


	os.system('rm -rf '+VISINAME+'.selfcal')
	split(vis=VISINAME, outputvis=VISINAME+'.selfcal', field='Arp_220', spw='0,1,2,3', datacolumn='corrected')


if DoContSubtract:

	print "Doing uv continuum subtraction"
	os.system('rm -rf '+VISINAME+'.selfcal.contsub')
	uvcontsub(vis=VISINAME+'.selfcal', field='Arp_220', fitspw=spw4Cont, spw='0,1,2,3', combine='spw', fitorder=0)



if DoCleanAllspw:     # This will clean all the spectral windows.

    ChanAve=[20,40,40,20]  # Averaging per spectral window. spw 1 and 2 have twice better spectral resolution. Here we bring them all to the same spectral resolution of ~20 MHz.
    ii=0	
    for spw in ['0', '1', '2', '3']:  # Will loop over all spectral windows
        print "CLEAN", SOUNAME, TUNING, "spw",spw
        imagename=SOUNAME+'-'+TUNING+'-spw'+spw
        os.system('rm -rf '+imagename+'*')
        tclean(vis=VISINAME+'.selfcal.contsub',
	       datacolumn='data',
               spw=spw,
               imagename=imagename,
               field="Arp_220",
               cell=cell,
               imsize=imsize,
               outframe='bary',
               niter=1000,
               interactive=False,
               mask=maskArp220,
               specmode='cube',
               start=0,
	       nchan=-1,
               width=ChanAve[ii],   # Will apply the channel averaging as set in ChanAve
               threshold='3mJy',
               weighting='briggs',
               robust=0.5,
               interpolation='linear',
               phasecenter=phasecenterArp220,
	       pbcor=True)
	ii += 1
        os.system('rm -f '+imagename+'.image.pbcor.fits')
        exportfits(imagename=imagename+'.image.pbcor',fitsimage=imagename+'.image.pbcor.fits')


if DoCleanLine:

	print "Cleaning line in velocity mode"
	zz = 0.01812   # Redshift of Arp220

	# LineList consists in LineName, Line Rest Frequency (GHz), and the spw ID where the line is:
	LineList=[['HNC_2-1',181.3247580,'1'],['H2O_313-220',183.3101170,'0'],['CS_4-3',195.9542109,'3']]
                # there some other weaker (non-identified) lines around sky frequencies:
		# 179.3, 180.7 GHz (spw0), 190.1 GHz (spw2), 191.7, 193.0 GHz (spw3).


	for ii in range(len(LineList)):  # will loop over all lines listed in LineList
	        print "CLEAN Line", ii+1, "out of", len(LineList), SOUNAME, LineList[ii][0], "spw",LineList[ii][2]
        	imagename=SOUNAME+'-'+LineList[ii][0]
		print "Redshifted frequency=",LineList[ii][1]/(1+zz)
        	os.system('rm -rf '+imagename+'*')
        	tclean(vis=VISINAME+'.selfcal.contsub',
		       datacolumn='data',	
        	       spw=LineList[ii][2],
        	       imagename=imagename,
        	       field="Arp_220",
        	       cell=cell,
        	       imsize=imsize,
        	       outframe='bary',
        	       niter=1000,
        	       interactive=False,
        	       mask=maskArp220,
        	       specmode='cube',
        	       start='-500km/s',
		       nchan=50,
        	       width='20km/s',
        	       threshold='3mJy',
        	       weighting='briggs',
        	       robust=0.5,
                       restoringbeam='common',
        	       interpolation='linear',
        	       phasecenter=phasecenterArp220,
		       restfreq=str(LineList[ii][1]/(1+zz))+'GHz',
		       pbcor=True)
        	os.system('rm -f '+imagename+'.image.pbcor.fits')
        	exportfits(imagename=imagename+'.image.pbcor',fitsimage=imagename+'.image.pbcor.fits')
		Stat=imstat(imagename=imagename+'.image.pbcor.fits',box='190,350,400,500',chans='2~38')
		print "Rms=", Stat['rms'][0]*1e3,'mJy'

		print "Doing moment maps"
		os.system('rm -rf '+imagename+'.mom*')
		immoments(imagename=imagename+'.image.pbcor', moments=[0], chans='2~38', includepix=[0.0, 999.9], outfile=imagename+'.mom0')
		immoments(imagename=imagename+'.image.pbcor', moments=[1], chans='2~38', includepix=[0.017, 999.9], outfile=imagename+'.mom1')
		immoments(imagename=imagename+'.image.pbcor', moments=[2], chans='2~38', includepix=[0.017, 999.9], outfile=imagename+'.mom2')
                exportfits(imagename=imagename+'.mom0',fitsimage=imagename+'.mom0.fits')
                exportfits(imagename=imagename+'.mom1',fitsimage=imagename+'.mom1.fits')
                exportfits(imagename=imagename+'.mom2',fitsimage=imagename+'.mom2.fits')




if False:

	print "CLEAN Continuum before selfcal"
       	imagename=SOUNAME+'-B5-cont-beforeSelfCal'
        os.system('rm -rf '+imagename+'*')
        tclean(	vis=VISINAME,
		datacolumn='data',
      		spw=spw4Cont,
        	imagename=imagename,
        	field="Arp_220",
        	cell=cell,
        	imsize=imsize,
        	outframe='bary',
        	niter=1000,
        	interactive=False,
        	mask=maskArp220,
        	specmode='mfs',      
        	threshold='0.5mJy',
        	weighting='briggs',
        	robust=0.5,
        	interpolation='linear',
        	phasecenter=phasecenterArp220,
		pbcor=True)
        os.system('rm -f '+imagename+'.fits')
        exportfits(imagename=imagename+'.image.pbcor',fitsimage=imagename+'.image.pbcor.fits')

	CheckSouFit=imfit(imagename=imagename+'.image.pbcor.fits',box='290,290,310,310')
	print "Fond flux density=", CheckSouFit['deconvolved']['component0']['flux']['value'][0]*1e3,'mJy'
	Stat=imstat(imagename=imagename+'.fits',box='190,350,400,500')
	print "Rms=", Stat['rms'][0]*1e3,'mJy'
	print "SNR=", CheckSouFit['deconvolved']['component0']['flux']['value'][0]/Stat['rms'][0]


if DoCleanCont:

	print "CLEAN Continuum after selfcal"
       	imagename=SOUNAME+'-B5-cont-afterSelfCal'
        os.system('rm -rf '+imagename+'*')
        tclean(	vis=VISINAME+'.selfcal',
		datacolumn='data',	
      		spw=spw4Cont,
        	imagename=imagename,
        	field="Arp_220",
        	cell=cell,
        	imsize=imsize,
        	outframe='bary',
        	niter=1000,
        	interactive=False,
        	mask=maskArp220,
        	specmode='mfs',      
        	threshold='0.2mJy',
        	weighting='briggs',
        	robust=0.5,
        	interpolation='linear',
        	phasecenter=phasecenterArp220,
		pbcor=True)
        os.system('rm -f '+imagename+'.fits')
        exportfits(imagename=imagename+'.image.pbcor',fitsimage=imagename+'.image.pbcor.fits')

	CheckSouFit=imfit(imagename=imagename+'.image.pbcor.fits',box='290,290,310,310')
	print "Found flux density=", CheckSouFit['deconvolved']['component0']['flux']['value'][0]*1e3,'mJy'
	Stat=imstat(imagename=imagename+'.image',box='190,350,400,500')
	print "Rms=", Stat['rms'][0]*1e3,'mJy'
	print "SNR=", CheckSouFit['deconvolved']['component0']['flux']['value'][0]/Stat['rms'][0]
        



if DoCleanCheckSou: # Clean check source J1532+2344

	for spw in ['0', '1', '2', '3']:
		print "CLEAN check source spw",spw
	        imagename='CheckSource-spw'+spw
	        os.system('rm -rf '+imagename+'*')
		mask_CheckSou=['circle[ [15h32m46.3464,+23d44m05.28], 2.0arcsec]']
	        tclean(vis=VISINAME,
		       datacolumn='data',	
	               spw=spw,
	               imagename=imagename,
	               field='J1532+2344',
	               cell=cell,
	               imsize=imsize,
	               outframe='bary',
	               niter=1000,
	               interactive=False,
	               mask=mask_CheckSou,
	               specmode='mfs',
	               threshold='1mJy',
	               weighting='briggs',
	               robust=0.5,
	               interpolation='linear',
	               phasecenter='J2000 15h32m46.3464 +23d44m05.28')
	        os.system('rm -f '+imagename+'.fits')
	        exportfits(imagename=imagename+'.image',fitsimage=imagename+'.fits')
		CheckSouFit=imfit(imagename=imagename+'.fits',box='290,290,310,310')
		print "Fond flux density=", CheckSouFit['deconvolved']['component0']['flux']['value'][0]*1e3,'mJy'
		CheckSouStat=imstat(imagename=imagename+'.fits',box='190,345,400,500')
		print "Rms=", CheckSouStat['rms'][0]*1e3,'mJy'







